from typing import Any as _Any, override as _override

from leads.constant import SystemLiteral
from leads.context import Context
from leads.event import InterventionEvent, InterventionExitEvent
from leads.plugin.plugin import Plugin


def do_dtcs(context: Context,
            front_wheel_speed: float,
            rear_wheel_speed: float) -> InterventionEvent:
    if front_wheel_speed < rear_wheel_speed:
        context.overwrite_throttle(0)
        return InterventionEvent(context, SystemLiteral.DTCS, front_wheel_speed, rear_wheel_speed)
    return InterventionExitEvent(context, SystemLiteral.DTCS, front_wheel_speed, rear_wheel_speed)


class DTCS(Plugin):
    def __init__(self) -> None:
        super(DTCS, self).__init__(("front_wheel_speed", "rear_wheel_speed"))

    @_override
    def pre_update(self, context: Context, kwargs: dict[str, _Any]) -> None:
        context.intervene(do_dtcs(context, kwargs["front_wheel_speed"], kwargs["rear_wheel_speed"]))



from typing import Any as _Any, override as _override

from leads.constant import SystemLiteral
from leads.context import Context
from leads.event import InterventionEvent, InterventionExitEvent
from leads.plugin.plugin import Plugin


def do_abs(context: Context,
           front_wheel_speed: float,
           rear_wheel_speed: float) -> InterventionEvent:
    if front_wheel_speed > rear_wheel_speed:
        context.overwrite_brake(0)
        return InterventionEvent(context, SystemLiteral.ABS, front_wheel_speed, rear_wheel_speed)
    return InterventionExitEvent(context, SystemLiteral.ABS, front_wheel_speed, rear_wheel_speed)


class ABS(Plugin):
    def __init__(self) -> None:
        super(ABS, self).__init__(("front_wheel_speed", "rear_wheel_speed"))

    @_override
    def pre_update(self, context: Context, kwargs: dict[str, _Any]) -> None:
        context.intervene(do_abs(context, kwargs["front_wheel_speed"], kwargs["rear_wheel_speed"]))



from enum import StrEnum as _StrEnum
from typing import Callable as _Callable, override as _override

from PIL import Image as _Image
from customtkinter import CTkImage as _CTkImage

from leads import require_config as _require_config
from leads_gui.system import _ASSETS_PATH

_ICONS_PATH: str = _ASSETS_PATH + "/icons"


class Color(_StrEnum):
    BLACK: str = "black"
    WHITE: str = "white"
    RED: str = "red"


class _Icon(_Callable[[int, Color | None], _CTkImage]):
    def __init__(self, name: str) -> None:
        self._name: str = name

    def load_source(self, color: Color) -> _Image:
        return _Image.open(f"{_ICONS_PATH}/{self._name}-{color}.png")

    def __call__(self, size: int | None = None, color: Color | None = None) -> _CTkImage:
        if size is None:
            size = _require_config().font_size_medium
        return _CTkImage(self.load_source(color if color else Color.BLACK),
                         None if color else self.load_source(Color.WHITE),
                         size=(size, size))


class _Logo(_Icon):
    def __init__(self) -> None:
        super().__init__("logo")

    @_override
    def load_source(self, color: Color) -> _Image:
        return _Image.open(_ASSETS_PATH + "/logo.png")

    @_override
    def __call__(self, size: int | None = None, color: Color | None = None) -> _CTkImage:
        if size is None:
            size = _require_config().font_size_medium
        return _CTkImage(self.load_source(Color.BLACK), size=(size, size))


Logo: _Logo = _Logo()

Battery: _Icon = _Icon("battery")
Brake: _Icon = _Icon("brake")
Car: _Icon = _Icon("car")
ESC: _Icon = _Icon("esc")
Engine: _Icon = _Icon("engine")
Hazard: _Icon = _Icon("hazard")
HighBeam: _Icon = _Icon("high-beam")
Left: _Icon = _Icon("left")
Light: _Icon = _Icon("light")
Motor: _Icon = _Icon("motor")
Right: _Icon = _Icon("right")
Satellite: _Icon = _Icon("satellite")
Speed: _Icon = _Icon("speed")
Stopwatch: _Icon = _Icon("stopwatch")



from typing import override as _override, Iterator as _Iterator, Any as _Any

from leads import Controller as _Controller, DataContainer as _DataContainer
from leads.data_persistence import Dataset as _Dataset


class ReplayController(_Controller):
    def __init__(self, dataset: _Dataset) -> None:
        super().__init__()
        self._dataset: _Dataset = dataset
        self._iterator: _Iterator[dict[str, _Any]] = iter(dataset)

    @_override
    def initialize(self, *parent_tags: str) -> None:
        super().initialize(*parent_tags)
        self._dataset.load()

    @_override
    def read(self) -> _DataContainer:
        try:
            d = next(self._iterator)
        except StopIteration:
            return _DataContainer()
        t = d.pop("t")
        dc = _DataContainer(**d)
        dc._time_stamp = t
        return dc
